@@ -5,7 +5,7 @@ L{ViewManager}.
 @author: Peter Parente
 @author: Pete Brunet
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
@@ -14,7 +14,8 @@ this distribution, and is available at
 U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
-import Base, Constants
+import Base
+import AEConstants
 from LSRInterfaces import *
 
 class ViewChange(Base.AccessEngineEvent):
@@ -22,30 +23,17 @@ class ViewChange(Base.AccessEngineEvent):
   Event that fires when the L{ViewManager} is creating or updating its view.
   
   This class registers its name and whether it should be monitored by default
-  in an L{AEMonitor} using the L{Constants.registerEventType} function when
+  in an L{AEMonitor} using the L{Base.registerEventType} function when
   this module is first imported. An L{AEMonitor} can use this information to
   build its menus.
 
-  @ivar gained: Type of view change: True if gained, False if lost, None if 
-    mutated
+  @ivar gained: Type of view change
   @type gained: boolean
   @ivar title: Name of the root element of the new view (e.g. title of the
     foreground window)
   @type title: string
-  @cvar GAINED: View was gained by activating it
-  @type GAINED: integer
-  @cvar LOST: View was lost by deactivating it
-  @type LOST: integer
-  @cvar STARTUP: View was identified at LSR startup
-  @type STARTUP: integer
-  @cvar FIRST_GAINED: View was gained by activating it during LSR startup
-  @type FIRST_GAINED: integer
   '''
-  LOST = 0
-  GAINED = 1 
-  STARTUP = 2
-  FIRST_GAINED = 3
-  Constants.registerEventType('ViewChange', True)
+  Base.registerEventType('ViewChange', True)
   def __init__(self, por, gained, **kwargs):
     '''
     Stores the type of view change and intializes the title attribute to
@@ -53,10 +41,11 @@ class ViewChange(Base.AccessEngineEvent):
 
     @param por: Point-of-regard to the accessible at the root of the new view
     @type por: L{POR}    
-    @param gained: One of the integer class variables of this class
+    @param gained: Type of view change
     @type gained: integer
     '''
-    focused = gained in (ViewChange.GAINED, ViewChange.FIRST_GAINED)
+    focused = gained in (AEConstants.EVENT_VIEW_GAINED, 
+                         AEConstants.EVENT_VIEW_FIRST_GAINED)
     Base.AccessEngineEvent.__init__(self, focused=focused, **kwargs)
     self.por = por
     self.gained = gained
@@ -71,13 +60,13 @@ class ViewChange(Base.AccessEngineEvent):
     @rtype: string
     '''
     name = Base.AccessEngineEvent.__str__(self)
-    if self.gained == self.GAINED:
+    if self.gained == AEConstants.EVENT_VIEW_GAINED:
       action = 'gained'
-    elif self.gained == self.LOST:
+    elif self.gained == AEConstants.EVENT_VIEW_LOST:
       action = 'lost'
-    elif self.gained == self.FIRST_GAINED:
+    elif self.gained == AEConstants.EVENT_VIEW_FIRST_GAINED:
       action = 'first gained'
-    elif self.gained == self.STARTUP:
+    elif self.gained == AEConstants.EVENT_VIEW_STARTUP:
       action = 'startup'
     return '%s:\n\tPOR: %s\n\ttitle: %s\n\taction: %s' % (name, self.por,
                                                           self.title, action)
@@ -106,7 +95,8 @@ class ViewChange(Base.AccessEngineEvent):
     except LookupError:
       # view is dead
       return True
-    if self.gained in (self.GAINED, self.FIRST_GAINED):
+    if self.gained in (AEConstants.EVENT_VIEW_GAINED, 
+                       AEConstants.EVENT_VIEW_FIRST_GAINED):
       # get information about the accessible's application
       try:
         aid = ai.getAccAppID()
@@ -114,8 +104,14 @@ class ViewChange(Base.AccessEngineEvent):
       except LookupError:
         # view is dead
         return True
-      # ask the tier_manager to switch tiers first
+      # inform the view manager of the AE view change
+      view_manager.setAEView(self.por)
+      # ask the tier manager to switch tiers first
       tier_manager.switchTier(aname, aid, self.por)
+    elif self.gained == AEConstants.EVENT_VIEW_LOST:
+      if self.por == view_manager.getAEView():
+        # only unset if the view being deactivated is still the active view
+        view_manager.setAEView(None)
     # then process the view change event
     tier_manager.manageEvent(self)
     return True
