@@ -254,8 +254,9 @@ class Setting(object):
   @type _value: object
   @ivar _cached: Cached value from the last L{save} invocation
   @type _cached: object
-  @ivar _observers: List of callables to notify on a value change
-  @type _observers: list
+  @ivar _observers: Callables to notify on a value change mapped to their
+    arguments
+  @type _observers: dictionary
   '''
   def __init__(self, state, name, default, label, description, persist):
     '''
@@ -269,12 +270,23 @@ class Setting(object):
     self.persist = persist
     self._value = default
     self._cached = None
-    self._observers = []
+    self._observers = {}
     
     # register the state object as a listener for value changes so it can 
     # update its dirty set
     self.addObserver(self.state._makeDirty)
     
+  def update(self, setting):
+    '''
+    Updates this setting with the default value and current value of another
+    setting of the same kind.
+    
+    @param setting: Setting of the same kind as this one
+    @type setting: L{Setting}
+    '''
+    self.default = setting.default
+    self.value = setting.value
+    
   def copy(self):
     '''
     Makes a shallow copy of this object.
@@ -313,13 +325,13 @@ class Setting(object):
     indicated by a ReferenceError, removes it from the list of observers. 
     '''
     dead = []
-    for ob, args in self._observers:
+    for ob, args in self._observers.items():
       try:
         ob(self.state, self, *args)
       except ReferenceError:
-        dead.append((ob, args))
+        dead.append(ob)
     # remove dead observer proxies
-    map(self._observers.remove, dead)
+    map(self._observers.pop, dead)
          
   def addObserver(self, ob, *args):
     '''
@@ -343,7 +355,7 @@ class Setting(object):
         weak_args.append(weakref.proxy(a))
       except TypeError:
         weak_args.append(a)
-    self._observers.append((Proxy(ob), weak_args))
+    self._observers[Proxy(ob)] = weak_args
   
   def removeObserver(self, ob):
     '''
@@ -352,10 +364,10 @@ class Setting(object):
     
     @param ob: The observer
     @type ob: callable
-    @raise ValueError: When the given callable is not an observer of this
+    @raise KeyError: When the given callable is not an observer of this
       setting
     '''
-    self._observers.remove(Proxy(ob))
+    del self._observers[Proxy(ob)]
     
   def clearObservers(self):
     '''
@@ -443,6 +455,17 @@ class FilenameSetting(StringSetting):
     Setting.__init__(self, state, name, default, label, description, persist)
     self.relative_path = relative_path
     
+  def update(self, setting):
+    '''
+    Updates the relative path in addition to everything done in the parent
+    class method.
+    
+    @param setting: Setting of the same kind as this one
+    @type setting: L{FilenameSetting}
+    '''
+    super(FilenameSetting, self).update(setting)
+    self.relative_path = setting.relative_path
+    
   def _getValue(self):
     '''
     Gets the filename as a string.
@@ -488,6 +511,19 @@ class NumericSetting(Setting):
     self.max = max
     self.precision = precision
     
+  def update(self, setting):
+    '''
+    Updates the min, max, and precision in addition to everything done in the 
+    parent class method.
+    
+    @param setting: Setting of the same kind as this one
+    @type setting: L{NumericSetting}
+    '''
+    super(NumericSetting, self).update(setting)
+    self.min = setting.min
+    self.max = setting.max
+    self.precision = setting.precision
+    
   def _fixBounds(self, val, min, max, prec):
     '''
     Snaps values to the specific range and precision.
@@ -578,6 +614,26 @@ class ChoiceSetting(Setting):
     Setting.__init__(self, state, name, default, label, description, persist)
     self.values = choices
     
+  def addChoice(self, choice):
+    '''
+    Add a new choice possibility.
+    
+    @param choice: New option
+    @type choice: string
+    '''
+    self.values.append(choice)
+    
+  def update(self, setting):
+    '''    
+    Updates the choices in addition to everything done in the parent class
+    method.
+    
+    @param setting: Setting of the same kind as this one
+    @type setting: L{ChoiceSetting}
+    '''
+    super(ChoiceSetting, self).update(setting)
+    self.values = setting.values
+    
   def _setValue(self, val):
     '''
     Stores the given choice value as the active selection. If the value is not
@@ -587,11 +643,21 @@ class ChoiceSetting(Setting):
     @type val: object
     '''
     if val not in self.values:
-      val = self.values[0]
+      val = self.default
     Setting._setValue(self, val)
     
   value = property(Setting._getValue, _setValue)
   
+  def unserialize(self, val):
+    '''
+    Does not bypass the fset property method so that when data is restored, all
+    invalid values are set back to the default.
+
+    @param val: Data value to store
+    @type val: object
+    '''
+    self.value = val
+  
 class EnumSetting(ChoiceSetting):
   '''
   Represents a one of many choice setting where text labels describe an 
@@ -609,6 +675,39 @@ class EnumSetting(ChoiceSetting):
     self.labels.sort()
     ChoiceSetting.__init__(self, state, name, default, label, description, 
                            persist, [choices[key] for key in self.labels])
+    
+  def getLabel(self, choice):
+    '''
+    Gets the label for a given choice.
+    
+    @param choice: Value for which to retrieve the label
+    @type choice: object
+    @raise ValueError: When L{choice} is not a valid value
+    '''
+    return self.labels[self.values.index(choice)]
+    
+  def addChoice(self, label, choice):
+    '''
+    Add a new choice possibility with the given label.
+    
+    @param label: Human readable name
+    @type label: string
+    @param choice: New option
+    @type choice: object
+    '''
+    super(EnumSetting, self).addChoice(choice)
+    self.labels.append(label)
+    
+  def update(self, setting):
+    '''    
+    Updates the labels in addition to everything done in the parent class
+    method.
+    
+    @param setting: Setting of the same kind as this one
+    @type setting: L{EnumSetting}
+    '''
+    super(EnumSetting, self).update(setting)
+    self.labels = setting.labels
 
 class ColorSetting(Setting):
   '''
@@ -656,6 +755,18 @@ class RelNumericSetting(NumericSetting):
     self.parent = style.getDefault()
     self.rel_min = -(max-min)
     self.rel_max = max-min
+    
+  def update(self, setting):
+    '''    
+    Updates the relative min and max in addition to everything done in the 
+    parent class method.
+    
+    @param setting: Setting of the same kind as this one
+    @type setting: L{RelNumericSetting}
+    '''
+    super(RelNumericSetting, self).update(setting)
+    self.rel_min = setting.rel_min
+    self.rel_max = setting.rel_max
   
   def _getValue(self):
     '''
