@@ -46,7 +46,9 @@ AE_MAP = {ViewChange : ['window:activate', 'window:deactivate',
                          'object:state-changed:checked',
                          'object:state-changed:enabled',
                          'object:state-changed:animated',
-                         'object:state-changed:busy'],
+                         'object:state-changed:busy',
+                         'object:state-changed:visible',
+                         'object:state-changed:showing'],
           TableChange : ['object:row-inserted',
                          'object:row-deleted',
                          'object:row-reordered',
@@ -216,7 +218,8 @@ class DefaultEventHandlerAdapter(Adapter):
           # a redundant object role
           return
         # make a first guess as to whether the object is focused or not
-        focused = (ss.contains(Constants.STATE_FOCUSED) or
+        focused = (DefaultEventHandlerAdapter.last_focus == event.source or 
+                   #ss.contains(Constants.STATE_FOCUSED) or
                    ss.contains(Constants.STATE_ACTIVE))
       except AttributeError:
         # abort immediately if the CORBA object is dead
