@@ -46,12 +46,12 @@ class DeviceManager(object):
   @type acc_eng: L{AccessEngine}
   @ivar marks: Maps index markers on devices to 
   @type marks: dictionary
-  @ivar temp_dev: List of L{AEOutput} and L{AEInput} devices last removed in 
+  @ivar temp_devs: List of L{AEOutput} and L{AEInput} devices last removed in 
     L{unloadDevices} and held until the next call of L{loadDevices}. The 
     purpose of this list is to ensure strong references to all devices exist,
     and thus weak references are not destroyed, until the the L{DeviceManager}
     is repopulated with devices.
-  @type temp_dev: list
+  @type temp_devs: list
   '''
   def __init__(self, acc_eng):
     '''
@@ -68,7 +68,7 @@ class DeviceManager(object):
     self.event_manager = None
     self.settings_manager = None
     self.marks = {}
-    self.temp_out = []
+    self.temp_devs = []
 
   def init(self, event_man, sett_man, **kwargs):
     '''
@@ -120,7 +120,7 @@ class DeviceManager(object):
   def unloadDevices(self):
     '''
     Unloads all devices from L{out_devs} and L{in_devs}, but keeps strong 
-    references to all objects in L{temp_out} until L{loadDevices} is called 
+    references to all objects in L{temp_devs} until L{loadDevices} is called 
     again. Keeping strong references ensures weak references in L{Perk}s 
     continue to exist until the L{DeviceManager} has new devices available 
     for I/O.
@@ -178,13 +178,20 @@ class DeviceManager(object):
     # quit if no device specified
     if dev is None: return
     if sem is None or layer is None:
-      return dev.send(name, value, None)
+      try:
+        return dev.send(name, value, None)
+      except NotImplementedError:
+        return None
     else:
       style = dev.getStyle((sem, layer))
       # respect muting
       if style.Mute: return
       # send the data
-      rv = dev.send(name, value, style)
+      try:
+        rv = dev.send(name, value, style)
+      except NotImplementedError:
+        # no return value when not implemented
+        rv = None
       # mark the style as clean
       style.makeClean()
       return rv
