@@ -9,8 +9,9 @@ code constants used to register filters on that device.
 @type def_keymap: gtk.gdk.Keymap
 
 @author: Peter Parente
+@author: Scott Haeger
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
@@ -18,13 +19,15 @@ available under the terms of the BSD license which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-import AEInput, pyLinAcc
+import AEInput, pyLinAcc, AEConstants
 # need pygtk to map keycodes to keynames
 import pygtk
 pygtk.require('2.0')
 import gtk.gdk as gdk
 from i18n import _
 
+DEBUG = False
+
 def_keymap = gdk.keymap_get_default()
 MAX_KEY_CHORD = 3
 
@@ -59,9 +62,9 @@ class Keyboard(AEInput.SystemInput):
 
   Instances of L{Keyboard} impose some constraints on the filters that can be
   registered. A filter can contain at most one L{AEInput.Gesture} and that
-  L{AEInput.Gesture} may have at most L{MAX_KEY_CHORD} key codes, must start
-  with at least one modifier key as registered via L{addModifier}, and must end 
-  with and contain exactly one non-modifier key code.
+  L{AEInput.Gesture} may have at most L{MAX_KEY_CHORD} key codes and at least 
+  one. The filtered gesture may start with zero, one, or two modifiers 
+  registered via L{addModifier} and must end with exactly one non-modifier.
 
   The L{Keyboard} device is robust against typomatic repeats and more than
   L{MAX_KEY_CHORD} sustained presses. Typomatic repeats are ignored so
@@ -89,12 +92,17 @@ class Keyboard(AEInput.SystemInput):
   @type state: callable
   @ivar last_press: Was the last event a press (True) or release (False)?
   @type last_press: boolean
-  @ivar stateful: Last stateful key press
-  @type stateful: integer
   @ivar LOCK_KEYS: Key codes for keys that can be locked in a particular state,
     namely L{AEK_CAPS_LOCK}, L{AEK_SCROLL_LOCK}, and L{AEK_NUM_LOCK}. These
     are constant.
   @type LOCK_KEYS: list of integer
+  @ivar ACC_KEYS: Key codes for keys that should never be consumed because they
+    activate accessibility features such as slow keys or sticky keys
+  @type ACC_KEYS: list of integer
+  @ivar EATEN_GESTURES: List of gestures commonly eaten by a party higher than
+    this device in the AT-SPI event chain (e.g. the window manager). This 
+    device should not synthesize these gestures when detected.
+  @type EATEN_GESTURES: list of L{AEInput.Gesture}
   '''
   def init(self):
     '''
@@ -102,12 +110,11 @@ class Keyboard(AEInput.SystemInput):
     reference to None. Creates an empty L{AEInput.Gesture} that will be used to
     buffer incoming keystrokes until the L{AEInput.Gesture} definitely matches
     or does not match one of the held filters. Registers for key press and 
-    release events via L{pyLinacc}. Sets the filtering mode to handled by
+    release events via L{pyLinAcc}. Sets the filtering mode to handled by
     default. Also registers for window activate events to account for missing 
     keys caused by system wide hotkey presses.
     '''
     self.filters = {}
-    self.stateful = None
     self.event_manager = pyLinAcc.Event.Manager()
     self.curr_gesture = AEInput.Gesture(self)
     self.state = self._stateInvalid
@@ -116,11 +123,16 @@ class Keyboard(AEInput.SystemInput):
     self.last_press = False
     self.LOCK_KEYS = [self.AEK_CAPS_LOCK, self.AEK_SCROLL_LOCK, 
                       self.AEK_NUM_LOCK]
+    self.ACC_KEYS = [self.AEK_SHIFT_L, self.AEK_SHIFT_R]
+    self.EATEN_GESTURES = [
+      AEInput.Gesture(self,[self.AEK_ALT_L, self.AEK_F4]),
+      AEInput.Gesture(self,[self.AEK_ALT_R, self.AEK_F4])
+    ]
     
     self.event_manager.addClient(self._onKeyPressed, 'keyboard:press')
     self.event_manager.addClient(self._onKeyReleased, 'keyboard:release')
     self.event_manager.addClient(self._onForegroundChange, 'window:activate')
-    self.setFilterMode(AEInput.Constants.FILTER_HANDLED)
+    self.setFilterMode(AEConstants.FILTER_HANDLED)
 
   def close(self):
     '''
@@ -132,32 +144,41 @@ class Keyboard(AEInput.SystemInput):
     self.event_manager.removeClient(self._onKeyPressed, 'keyboard:press')
     self.event_manager.removeClient(self._onKeyReleased, 'keyboard:release')
     self.event_manager.removeClient(self._onForegroundChange,'window:activate')
-    self.setFilterMode(AEInput.Constants.FILTER_NONE)
+    self.setFilterMode(AEConstants.FILTER_NONE)
+    
+  def getCapabilities(self):
+    '''
+    Gets the list of capabilities provided by this device, namely 'keyboard.'
+    
+    @return: 'keyboard' as the only capability of this device.
+    @rtype: list of string
+    '''
+    return ['keyboard']
 
   def addFilter(self, key_filter):
     '''
     Adds a L{AEInput.GestureList} to the collection of filters. The fire
     L{AEInput.Gesture} in the L{AEInput.GestureList} will be used when the mode
-    is set to L{AEInput.Constants.FILTER_HANDLED} to prevent keys consumed by
-    LSR from reaching the foreground application.
+    is set to filter handled to prevent keys consumed by LSR from reaching the 
+    foreground application.
 
     The L{Keyboard} input device only supports one L{AEInput.Gesture} per
-    filter, not a sequence of L{AEInput.Gesture}s, and only one non-modifier key
-    code per L{AEInput.Gesture} chord. Any L{AEInput.GestureList} breaking these
-    rules will cause this method to raise a ValueError.
+    filter, not a sequence of L{AEInput.Gesture}s, and only one non-modifier
+    key code per L{AEInput.Gesture} chord. Any L{AEInput.GestureList} breaking
+    these rules will cause this method to raise a ValueError.
 
     @param key_filter: Filter to add to the filter list
     @type key_filter: L{AEInput.GestureList}
-    @raise ValueError: When the given L{AEInput.GestureList} does not conform to
-      the rules set forth above
+    @raise ValueError: When the given L{AEInput.GestureList} does not conform 
+      to the rules set forth above
     '''
     # make sure there is only one Gesture in the list
     if key_filter.getNumGestures() != 1:
-      raise ValueError('Only one Gesture allowed in a Keyboard filter')
+      raise ValueError('only one Gesture allowed in a Keyboard filter')
     gesture = key_filter.getGestureAt(0)
     # make sure there is at least one keystroke in the Gesture
     if gesture.getNumActions() < 1:
-      raise ValueError('At least one key code required')
+      raise ValueError('at least one key code required')
     # make sure there is only one non-modifier key code
     count = 0
     for code in gesture:
@@ -166,7 +187,7 @@ class Keyboard(AEInput.SystemInput):
       except KeyError:
         count += 1
     if count > 1:
-      raise ValueError('At most one non-modifier key allowed')
+      raise ValueError('at most one non-modifier key allowed')
     # add the filter to the filters dictionary indexed by the first Gesture
     self.filters[gesture] = None
 
@@ -206,22 +227,6 @@ class Keyboard(AEInput.SystemInput):
     '''
     return _('Keyboard')
 
-  def sortGesture(self, gesture):
-    '''
-    Sorts the actions in the given L{AEInput.Gesture} and returns them as a list
-    of integers. The sort is done on a copy so the action codes held by the
-    provided L{AEInput.Gesture} are not touched.
-
-    @param gesture: L{AEInput.Gesture} to sort
-    @type gesture: L{AEInput.Gesture}
-    @return: Sorted list of action codes that may be wrapped in a new
-        L{AEInput.Gesture}
-    @rtype: list of integer
-    '''
-    codes = gesture.getActionCodes()
-    codes.sort()
-    return codes
-
   def resetState(self):
     '''
     Resets the state of the L{Keyboard} by clearing out the L{curr_gesture} and
@@ -230,6 +235,20 @@ class Keyboard(AEInput.SystemInput):
     self.curr_gesture.clearActionCodes()
     self.state = self._stateInvalid
     
+  def _debug(self, header, code, event, press):
+    if DEBUG:
+      if press:
+        print 'press:', code
+      else:
+        print 'release:', code
+      print header
+      print 'next state:', self.state
+      print 'consumed:', event.consume
+      print 'curr key:', str(self.curr_gesture)
+      print 'synth pressed:', str(self.synth_press)
+      print 'synth released:', str(self.synth_release)
+      print '----------------------------'
+    
   def _hasOneModifier(self, gesture):
     '''
     Peeks in the given gestures to see if it contains at least one modifier.
@@ -263,23 +282,24 @@ class Keyboard(AEInput.SystemInput):
     Synthesizes key strokes for the key codes in the given L{AEInput.Gesture}.
     The presses are noted in L{synth_press} and L{synth_release} so that they
     can be ignored when they are seen in L{_onKeyPressed} or L{_onKeyReleased}
-    to avoid duplicate processing.
+    to avoid duplicate processing. 
 
     @param press: Synthesize key presses (True) or releases (False)?
     @type press: boolean
     @param gesture: Gesture containing key codes to be pressed
     @type gesture: L{AEInput.Gesture}
     '''
-    # look at the key codes without making a copy
-    codes = gesture.peekActionCodes()
+    # see if the gesture is one of our eaten key combos
+    if gesture in self.EATEN_GESTURES:
+      # do nothing
+      return
+
     # generate all events
-    for code in codes:
-      self.event_manager.generateKeyEvent(code, press)
-      if press:
-        self.synth_press.append(code)
-      else:
-        self.synth_release.append(code)
-        
+    for code in gesture.peekActionCodes():
+      if code in self.LOCK_KEYS or code in self.ACC_KEYS:
+        continue
+      self._sendKey(code, press)
+
   def _sendKey(self, code, press):
     '''
     Synthesizes a single key stroke. The presses are noted in L{synth_press} 
@@ -296,7 +316,57 @@ class Keyboard(AEInput.SystemInput):
       self.synth_press.append(code)
     else:
       self.synth_release.append(code)
-
+      
+  def _consumeKey(self, event, press):
+    '''
+    Consumes the given event as long as it is not a special key used by other
+    system services for accessibility purposes such as shift for slow keys
+    and sticky keys.
+    
+    @param event: Keyboard event
+    @type event: L{pyLinAcc.Event.Event}
+    @param press: Press or release event?
+    @type press: boolean
+    '''
+    #if not press or (event.detail2 not in self.ACC_KEYS):
+    event.consume = True
+    if press and (event.detail2 in self.ACC_KEYS):
+      # allow accessibility keys to pass, but as synthesized events so later
+      # release events can be paired properly by X and not generate a second 
+      # press
+      self._sendKey(event.detail2, True)
+      
+  def _handleStateful(self, event, press):
+    '''
+    Handles the use of stateful keys such as caps lock, num lock, and scroll
+    lock as modifiers. When a lock key is pressed, this method immediately 
+    sets it back to its former state. When a lock key is released, this method
+    allows the change to pass but only if all of the conditions are met:
+    
+      - The lock key was the only key in the L{curr_gesture} since it was last
+        empty.
+      - The lock key is not a gesture itself.
+      - The lock key is currently the only key in the L{curr_gesture}.
+       
+    @param event: Keyboard event
+    @type event: L{pyLinAcc.Event.Event}
+    @param press: Was the event a press (True) or release (False)?
+    @type press: boolean
+    '''
+    code = event.detail2
+    if code in self.LOCK_KEYS:
+      if press:
+        # deactive lock immediately after it has been pressed
+        self._sendKey(code, True)
+      elif (self.last_press == True and
+            not self.filters.has_key(self.curr_gesture) and 
+            len(self.curr_gesture) == 1):
+        # reactivate lock if it is the only key pressed, wasn't part of a combo
+        # where another key was pressed and released, and is not a filter key
+        # itself
+        self._sendKey(code, True)
+        self._sendKey(code, False)
+      
   def _statePossible(self, event, press):
     '''
     Represents a state where the current key codes in L{curr_gesture} and the
@@ -323,28 +393,32 @@ class Keyboard(AEInput.SystemInput):
     '''
     if self.modifiers.has_key(event.detail2):
       # stay in this state and consume the key
-      event.consume = True
+      self._consumeKey(event, press)
       return self._statePossible
     elif self.filters.has_key(self.curr_gesture):
       # move to the match state and consume the key
-      event.consume = True
+      self._consumeKey(event, press)
       return self._stateMatch
     else:
       # update the os key state
       self._sendGesture(self.curr_gesture, True)
       # move to the invalid state and consume the current key
-      event.consume = True
+      self._consumeKey(event, press)
       return self._stateInvalid
 
   def _stateInvalid(self, event, press):
     '''
     Represents a state where the current key codes in L{curr_gesture} and the
-    next key event could not possibly match one of the registered L{filters}.
+    next key event is not likely to match one of the registered L{filters}.
     The actions in this state are as follows.
 
     If the event is a press resulting in the L{curr_gesture} having only
     L{modifiers} in it, we consume the press and transition to
     L{_statePossible}.
+    
+    If the event is a press resulting in the L{curr_gesture} exactly matching
+    one of the registered L{filters}, we consume the press transition to 
+    L{_stateMatch}.
 
     If the event is anything else, we allow the event to pass and stay in this
     state.
@@ -356,10 +430,19 @@ class Keyboard(AEInput.SystemInput):
     @return: Method representing the next state
     @rtype: callable
     '''
-    if press and self._hasOnlyModifiers(self.curr_gesture):
-      # consume and move to the possible match state
-      event.consume = True
-      return self._statePossible
+    if press:
+      if self._hasOnlyModifiers(self.curr_gesture):
+        # just a modifier, possibly a filter to match
+        self._consumeKey(event, press)
+        return self._statePossible
+      elif self.filters.has_key(self.curr_gesture):
+        # already a matched filter
+        self._consumeKey(event, press)
+        return self._stateMatch
+      elif len(self.synth_press):
+        # still waiting for presses to be synthesized, eat and synth
+        self._consumeKey(event, press)
+        self._sendKey(event.detail2, True)
 
   def _stateMatch(self, event, press):
     '''
@@ -376,8 +459,8 @@ class Keyboard(AEInput.SystemInput):
     L{filters}, we synthesize key presses for all keys in L{curr_gesture},
     consume the current press, and transition to L{_stateInvalid}.
 
-    If the event is a release of one of the L{modifiers}, we consume the release
-    and move to L{_stateAfterMatch}.
+    If the event is a release of one of the L{modifiers}, we consume the
+    release and move to L{_stateAfterMatch}.
 
     If the event is anything else (i.e. a non-modifier release), we consume the
     key and transition to L{_statePossible}.
@@ -391,30 +474,31 @@ class Keyboard(AEInput.SystemInput):
     '''
     if press:
       if self.filters.has_key(self.curr_gesture):
-        # stay in this state and do nothing, typomatic
+        # stay in this state and eat the key, typomatic
+        self._consumeKey(event, press)
         return self._stateMatch
       else:
         # send all key presses
         self._sendGesture(self.curr_gesture, True)
         # move to the invalid state and eat the key
-        event.consume = True
+        self._consumeKey(event, press)
         return self._stateInvalid
     elif self.modifiers.has_key(event.detail2):
       # move to the after match state and eat the key
-      event.consume = True
+      self._consumeKey(event, press)
       return self._stateAfterMatch
     else:
       # move to the possible state and eat the key
-      event.consume = True
+      self._consumeKey(event, press)
       return self._statePossible
     
   def _stateAfterMatch(self, event, press):
     '''    
-    Represents a state where the the keys in L{curr_gesture} just matched one of
-    the L{filters} and the next event was a release of one of the modifier keys
-    instead of the non-modifier. This state is needed to avoid typomatic repeat
-    presses of the still held non-modifier key. The actions in this state are as
-    follows.
+    Represents a state where the the keys in L{curr_gesture} just matched one
+    of the L{filters} and the next event was a release of one of the modifier
+    keys instead of the non-modifier. This state is needed to avoid typomatic
+    repeat presses of the still held non-modifier key. The actions in this
+    state are as follows.
     
     If the event is a release of a non-modifier and the L{curr_gesture} is now
     empty, we consume the release and move to L{_stateInvalid}.
@@ -432,7 +516,7 @@ class Keyboard(AEInput.SystemInput):
     '''
     if not press and not self.modifiers.has_key(event.detail2):
       # eat the key
-      event.consume = True
+      self._consumeKey(event, press)
       if self.curr_gesture.getNumActions() == 0:
         # move to state invalid
         return self._stateInvalid
@@ -441,7 +525,7 @@ class Keyboard(AEInput.SystemInput):
         return self._statePossible
     else:
       # stay in this state and eat the key
-      event.consume = True
+      self._consumeKey(event, press)
 
   def _onKeyPressed(self, event):
     '''
@@ -459,11 +543,18 @@ class Keyboard(AEInput.SystemInput):
     @type event: L{pyLinAcc.Event.Event}
     '''
     code = event.detail2
-    #print 'press:', event.detail2
     if len(self.synth_press) and event.detail2 == self.synth_press[0]:
       # ignore keys we know we synthesized
       self.synth_press.pop(0)
+      self._debug('ignored our press', code, event, True)
       return
+    elif code in self.synth_release:
+      # ignore presses for keys we're waiting to release
+      # do not account for ACC_KEYS, ignore everything
+      event.consume = True
+      self._debug('ignored our press before release', code, event, True)
+      return
+
     if code not in self.curr_gesture:
       # only count a non-typomatic press as a last press
       self.last_press = True
@@ -474,27 +565,13 @@ class Keyboard(AEInput.SystemInput):
         self.curr_gesture.addActionCode(code)
       except ValueError:
         # ignore the press if the gesture is full
+        self._debug('error adding', code, event, True)
         return
-
-    # treat stateful keys in a special manner if they're modifiers and another
-    # key has been pressed after one has been held
-    if (self.modifiers.has_key(code) and code in self.LOCK_KEYS):
-      self.stateful = code
-    if (len(self.curr_gesture) > 1 and self.stateful is not None):
-      # build a gesture with the stateful key press only in it
-      g = AEInput.Gesture(self, action_codes=[self.stateful])
-      self._sendGesture(g, True)
-      self.stateful = None
       
-    #print 'state:', self.state
-    #print 'curr key:', str(self.curr_gesture)
+    self._handleStateful(event, True)
+      
     self.state = self.state(event, True) or self.state
-    #print 'next state:', self.state
-    #print 'next key:', str(self.curr_gesture)
-    #print 'consumed:', event.consume
-    #print 'synth pressed:', str(self.synth_press)
-    #print 'synth released:', str(self.synth_release)
-    #print '----------------------------'
+    self._debug('state machine handled', code, event, True)
 
   def _onKeyReleased(self, event):
     '''
@@ -516,35 +593,41 @@ class Keyboard(AEInput.SystemInput):
     @type event: L{pyLinAcc.Event.Event}
     '''
     code = event.detail2
-    #print 'release:', event.detail2
     if len(self.synth_release) and code == self.synth_release[0]:
       # ignore keys we know we synthesized
       self.synth_release.pop(0)
+      try:
+        # take the released key out of the gesture
+        self.curr_gesture.removeActionCode(code)
+      except ValueError:
+        print 'error removing:', code
+      self._debug('ignored our release', code, event, False)
       return
-    elif code in self.synth_press:
-      # ignore releases for presses we're waiting to synth
+    elif len(self.synth_press):
+      # ignore releases when we're waiting to synth presses
+      self._consumeKey(event, False)
       # but make sure to synth our own release
       self._sendKey(code, False)
-      event.consume = True
+      self._debug('ignored release before press', code, event, False)
       return
     elif self.last_press:
       # make a copy of the current gesture before we modify it
       gesture = AEInput.Gesture(self, gesture=self.curr_gesture)
       # notify listeners of the gesture and its timestamp
       self._notifyInputListeners(gesture, event.any_data[3])
+
+    # do special handling for stateful keys
+    self._handleStateful(event, False)     
+      
     try:
+      # take the released key out of the gesture
       self.curr_gesture.removeActionCode(code)
     except ValueError:
+      self._debug('error removing', code, event, False)
       return
-    #print 'state:', self.state
-    #print 'curr key:', str(self.curr_gesture)
+    
     self.state = self.state(event, False) or self.state
-    #print 'next state:', self.state
-    #print 'next key:', str(self.curr_gesture)
-    #print 'consumed:', event.consume
-    #print 'synth pressed:', str(self.synth_press)
-    #print 'synth released:', str(self.synth_release)
-    #print '----------------------------'
+    self._debug('state machine handled', code, event, False)
     self.last_press = False
     
   def _onForegroundChange(self, event):
