@@ -7,17 +7,17 @@ Defines a class that extends L{pyLinAcc.Event.Manager} to support L{AEEvent}s.
 @author: Peter Parente
 @author: Pete Brunet
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import pyLinAcc
 import Queue, time, logging
-import AEEvent, AEMonitor, UIRegistrar
+import AEEvent, AEMonitor, UIRegistrar, AEConstants
 from LSRInterfaces import implements
 from UIRegistrar import MONITOR
 from i18n import _
@@ -90,8 +90,8 @@ class EventManager(pyLinAcc.Event.Manager):
     class close method.
     '''
     super(EventManager, self).close()
-    del self.raw_queue
-    del self.ae_queue
+    self.raw_queue = Queue.Queue()
+    self.ae_queue = Queue.Queue()
     self.monitors.clear()
 
   def addMonitors(self, *monitors):
@@ -137,23 +137,26 @@ class EventManager(pyLinAcc.Event.Manager):
       return event.execute(view_manager=self.view_manager,
                            tier_manager=self.tier_manager,
                            event_manager=self)
-    except Exception:
+    except LookupError:
+      # just ignore dead objects
+      return True
+    except Exception, e:
       # don't get bad events stuck in a loop
+      log.exception(str(e))
       return True
 
-  def _flushEvents(self):
+  def flushEvents(self):
     '''
     Flushes the event queue by destroying it and recreating it.
-
-    @warning: Not thread safe but easily changed using a lock
     '''
     self.ae_queue = Queue.Queue()
+    self.raw_queue = Queue.Queue()
 
   def postEvents(self, *events):
-    '''
+    '''    
     Add L{AEEvent}s to the queue. Events can be added by any part of the system
-    Events marked with L{AEEvent.IMMEDIATE} priority are executed now instead of
-    being added to the queue. Immediate execution is necessary for some kinds of
+    Events marked with immediate priority are executed now instead of being
+    added to the queue. Immediate execution is necessary for some kinds of
     events.
     
     @param events: Events to queue for dispatch
@@ -161,10 +164,11 @@ class EventManager(pyLinAcc.Event.Manager):
     '''
     for e in events:
       if e is not None:
-        if e.priority == AEEvent.IMMEDIATE:
+        if e.priority == AEConstants.EXEC_IMMEDIATE:
           # some events need to execute immediately (e.g. view update)
           if self._executeEvent(e) == False:
             # if it doesn't work now, try again later
+            log.debug('re-queuing immediate AEEvent')
             self.ae_queue.put_no_wait(e)
         else:
           self.ae_queue.put_nowait(e)
@@ -235,8 +239,10 @@ class EventManager(pyLinAcc.Event.Manager):
       if rv == False:
         # try to execute the event again later
         later.append(event)
+        log.debug('re-queuing AEEvent')
       elif rv != True:
         pass
     # put all events that failed to execute back in the queue for later
     map(self.postEvents, later)
     return True
+
