@@ -7,8 +7,9 @@ accessibles and handling their events.
 @author: Larry Weiss
 @author: Brett Clippingdale
 @author: Eirikur Hallgrimsson
+@author: Scott Haeger
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
@@ -272,8 +273,8 @@ class IAccessibleInfo(Interface):
    
   def getAccRoleName():
     '''
-    Gets the unicode accessible role name for the object implementing this 
-    interface.
+    Gets the localized unicode accessible role name for the object implementing
+    this interface.
     
     @return: Unicode role name
     @rtype: string
@@ -487,6 +488,17 @@ class IAccessibleInfo(Interface):
     '''
     pass
   
+  def getAccPosition(self):
+    '''
+    Gets the position of the accessible object, usually upper-left corner.
+    
+    @return: x,y coordinate
+    @rtype: 2-tuple integer
+    @raise NotImplementedError: When this accessible does not support the
+      Component interface
+    '''
+    pass
+  
   def getAccFloatValue():
     '''
     Gets the current floating point value of the subject
@@ -578,6 +590,17 @@ class IAccessibleInfo(Interface):
     @raise LookupError: When the accessible object is dead
     '''
     pass
+  
+  def getAccRole():
+    '''
+    Gets the unlocalized string role identifier for the object implementing 
+    this interface.
+    
+    @return: Role name
+    @rtype: string
+    @raise LookupError: When the accessible object is dead
+    '''
+    pass
 
   def hasAccRole(role):
     '''
@@ -591,6 +614,17 @@ class IAccessibleInfo(Interface):
     '''
     pass
   
+  def hasOneAccRole(*roles):
+    '''
+    Gets if the subject has any one of the given roles.
+    
+    @param roles: Names of the roles
+    @type roles: list of string
+    @return: Does the accessible have any one of the given roles?
+    @rtype: boolean
+    '''
+    pass
+  
   def hasAccState(state):
     '''
     Gets if the subject has the given state.
@@ -899,6 +933,18 @@ class IAccessibleAction(Interface):
     '''
     pass
   
+  def doMouseClickAtPOR(mouseevent):
+    '''
+    Performs a mouse event on the center of a given point of regard.  If the
+    accessible has a role of text, then set cursor at given location in por
+    
+    @param mouseevent: mouse event to perform
+    @type mouseevent: integer of type EVENT_SYNTHMOUSE*
+    @return: Indicator of whether the acc was clicked or not
+    @raise LookupError: When the accessible object is dead
+    '''
+    pass
+  
 class IEventHandler(Interface):
   '''
   Provides methods for handling accessible events.
@@ -927,7 +973,7 @@ class IEventHandler(Interface):
     @type collector: callable
     @param vm: Reference to the view manager that has information about the 
       current view and needs to store a reference to a new view
-    @rtype vm: L{ViewManager}
+    @type vm: L{ViewManager}
     '''
     pass
   
