@@ -42,7 +42,6 @@ from i18n import _
 from SettingsManager import SettingsManager
 from AEConstants import HOME_USER, HOME_DIR, PROG_VERSION, PROG_DATE, \
      TEMPLATE_DIR, DEFAULT_ASSOCIATIONS, DEFAULT_PROFILE
-from LSRInterfaces import implements
 
 # define a regular expression for doing word wrap on the command line
 wrap_rx=re.compile(u"([\u2e80-\uffff])", re.UNICODE)
@@ -280,33 +279,40 @@ def exportPath(options):
   
 def say(options):
   '''
-  Uses the first available L{AEOutput} device to output a string.
+  Uses the first available audio L{AEOutput} device to output a string.
   
   @param options: Options seen on the command line
   @type options: optparse.Values
   @return: Description of the action
   @rtype: string
   '''
-  reg = UIRegistrar
+  import DeviceManager, SettingsManager
+
   if options.profile is None:
     profile = DEFAULT_PROFILE
   else:
     profile = options.profile[0]
-  for dev in reg.loadAssociated(UIRegistrar.DEVICE, profile):
-    # find the first output device
-    if implements(dev, AEOutput.AEOutput):
-      # output the utf-8 encoded string seen on the command line
-      s = options.action_value
-      try:
-        dev.init()
-      except:
-        # try another device
-        continue
-      styles = dev.createDistinctStyles(1,1)
-      dev.sendStringSync(s, styles[0])
-      dev.close()
-      return _('Said: %s') % s
-  return _('No default output device')
+    
+  class DummyEngine(object):
+    # dummy engine object which has the one method needed by the device manager
+    def getProfile(self):
+      return profile
+    
+  # build a partially initialized device manager
+  dm = DeviceManager.DeviceManager(DummyEngine())
+  dm.settings_manager = SettingsManager.SettingsManager(profile)
+  dm.loadDevices()
+  
+  # get an audio output device
+  try:
+    dev = dm.getOutputByCaps(['audio'])
+  except Exception:
+    return _('No audio output device')
+  # output the utf-8 encoded string seen on the command line
+  s = unicode(options.action_value, 'utf-8')
+  dm.send(dev, AEConstants.CMD_STRING_SYNC, s, None, None)
+  dm.close()
+  return _('Said: %s') % s
     
 def show(options):
   '''
@@ -449,31 +455,42 @@ def getAction(option, opt_str, value, parser):
   parser.values.action = option.default
   parser.values.action_value = value
   
-def configA11y():
+def configA11y(options):
   '''
   Runs gconf to check if desktop accessibility support is enabled or not. If 
   not, enables it and asks the user to logout. 
   
-  @note: Current implementation is AT-SPI dependent
+  @note: Current implementation is GNOME, AT-SPI dependent
+  @param options: Options seen on the command line
+  @type options: optparse.Values
   @raise ValueError: When accessibility support is not enabled
   '''
-  # make sure desktop accessibility is enabled
+  if options.no_a11y:
+    return
   try:
     import gconf
   except ImportError:
     return
   cl = gconf.client_get_default()
   if not cl.get_bool('/desktop/gnome/interface/accessibility'):
-    cl.set_bool('/desktop/gnome/interface/accessibility', True)
-    raise ValueError(AEConstants.HELP_A11Y)
+    # use the UIRegistrar to load the A11yChooser
+    chooser = UIRegistrar.loadOne('A11yChooser')
+    chooser.init()
   
-def configGUI():
+def configGUI(options):
   '''  
   Configures the GUI toolkit with a default icon list for all LSR top level 
   windows. Also enables accessibility for this application.
 
   @note: Current implementation is gtk/gail dependent
   '''
+  if options.no_bridge:
+    # remove the bridge on purpose
+    os.environ['GTK_MODULES'] = ''
+  else:
+    # add GTK_MODULES to the environment so all GUIs are accessible
+    os.environ['GTK_MODULES'] = 'gail:atk-bridge'
+
   import pygtk
   pygtk.require('2.0')
   import gtk
@@ -487,9 +504,7 @@ def configGUI():
     pass
   else:
     gtk.window_set_default_icon_list(*icons)
-  # add GTK_MODULES to the environment so all GUIs are accessible
-  os.environ['GTK_MODULES'] = 'gail:atk-bridge'
-
+    
 def parseOptions():
   '''
   Parses the command line arguments based on the definition of the expected
@@ -558,6 +573,8 @@ def parseOptions():
                 dest='no_intro', help=AEConstants.HELP_NO_INTRO)
   op.add_option('--no-a11y', action='store_true', default=False, 
                 dest='no_a11y', help=AEConstants.HELP_NO_A11Y)
+  op.add_option('--no-bridge', action='store_true', default=False,
+                dest='no_bridge', help=AEConstants.HELP_NO_BRIDGE)
   op.add_option('--export-path', action='callback', callback=getAction,
                 default=exportPath, help=AEConstants.HELP_EXPORT_PATH)
   (options, args) = op.parse_args()
@@ -576,10 +593,6 @@ def main():
   # parse the command line options
   options = parseOptions()
 
-  # check if the default profiles need to be created
-  if not SettingsManager.hasDefaultProfiles():
-    initProfiles(None)
-
   try:
     # see if an action has been specified
     mtd = options.action
@@ -587,16 +600,21 @@ def main():
     # if not, run LSR
     mtd = run
 
+  # check if the default profiles need to be created
+  # but don't do it if we're initializing the global repo since we might be
+  # the root user
+  if mtd is not initGlobal and not SettingsManager.hasDefaultProfiles():
+    initProfiles(None)
+
   if mtd is not exportPath:
     # show the welcome message
     welcome()
 
   if mtd is run:
-    if not options.no_a11y:
-      # make sure accessibility is enabled on the desktop and for LSR
-      configA11y()
     # configure GUI toolkit
-    configGUI()
+    configGUI(options)
+    # make sure accessibility is enabled on the desktop and for LSR
+    configA11y(options)
     # set up the logging system
     LSRLog.configure(options.log_conf, options.log_level, options.log_channels,
                      options.log_file)
