@@ -244,7 +244,7 @@ class Perk(Task.Tools.All, UIElement.UIE):
       # raise a value error if there is already a command registered for this
       # sequence of keys in this Tier
       raise ValueError(
-        'Command already registered with action codes %s on device %s' % 
+        'command already registered with action codes %s on device %s' % 
         (codes, device.getName()))
     # add key list to our dictionary of commands and store the task to execute
     # as its value
@@ -425,12 +425,12 @@ class Perk(Task.Tools.All, UIElement.UIE):
     cid = id(chooser)
     task = self.keyed_tasks[cid]
     del self.keyed_tasks[cid]
-    # A chooser task is an odd beast because
-    # when a chooser dialog is getting destroyed, we want to unhook the task
-    # that was handling notifications for it from the Perk. However, we want
-    # that task to stay initialized for the next time the chooser is shown. 
-    # task.close()
-    # task.postClose()
+    
+    # We don't want to close the Task because technically it's not getting
+    # unregistered, just decoupled from the chooser it was managing. The Task
+    # must stay registered so that the chooser can be invoked again in the 
+    # future. Do not call Task.close() here!
+    
     self.tier.removeTaskRef(cid)
 
   def unregisterNamedTask(self, name):
@@ -580,7 +580,10 @@ class Perk(Task.Tools.All, UIElement.UIE):
     elif link == AEConstants.CHAIN_AFTER:
       return self.after_chains.get(target, [])[:]
     elif link == AEConstants.CHAIN_AROUND:
-      return self.around_chains
+      alist = []
+      if self.around_chains.has_key(target):
+        alist.append(self.around_chains[target])
+      return alist
   
   def linkTaskToChain(self, name, link, target):
     '''
@@ -678,4 +681,13 @@ class Perk(Task.Tools.All, UIElement.UIE):
         raise ValueError
 #      else:
         # remove a reference to this Perk from the Tier
-        #self.tier.removeChainRef(target, self)
\ No newline at end of file
+        #self.tier.removeChainRef(target, self)
+        
+  def getTaskNames(self):
+    '''
+    Gets the names of all L{Task}s registered in this L{Perk}.
+    
+    @return: All names
+    @rtype: list of string
+    '''
+    return self.named_tasks.keys()
\ No newline at end of file
