@@ -19,6 +19,9 @@ from POR import POR
 from i18n import bind, _
 from AEConstants import *
 
+# TODO:  remove.  for debug only
+import time
+
 # metadata describing this Perk
 __uie__ = dict(kind='perk', tier=None, all_tiers=True)
 
@@ -57,6 +60,9 @@ class BasicBraillePerkState(Perk.PerkState):
                 _('When set, Home, End of Line, and Start of Line commands '
                   'move caret to respective position. Otherwise, display '
                   'scrolls but caret remains in the same location.'))
+    self.newBool('BrailleReview', False, _('Track review keys?'), 
+                _('When set, reviewing is tracked by the Braille display.  '
+                  'Otherwise, display is not updated when reviewing.'))
    
   def getGroups(self):
     '''
@@ -78,6 +84,9 @@ class BasicBraillePerkState(Perk.PerkState):
     g.append('LeftPadding')
     g.append('RightPadding')
     
+    g = root.newGroup(_('Reviewing'))
+    g.append('BrailleReview')
+    
     return root
     
   
@@ -132,7 +141,8 @@ class BasicBraillePerk(Perk.Perk):
     self.registerTask(HandleLineEnd('braille line end'))
     self.registerTask(HandleHome('braille home'))
     self.registerTask(HandleReview('braille review'))
-    self.registerTask(HandlePointerToFocus('braille pointer to focus'))
+    self.registerTask(HandlePointerToFocus('braille pointer to por'))
+    self.registerTask(BrailleItemText('braille item text'))
                          
     #register braille device commands
     #TODO: send audible error
@@ -165,7 +175,7 @@ class BasicBraillePerk(Perk.Perk):
     self.chainTask('braille review', CHAIN_AFTER, 'review previous char')
     self.chainTask('braille review', CHAIN_AFTER, 'review current char')
     self.chainTask('braille review', CHAIN_AFTER, 'review next char')
-    self.chainTask('braille pointer to focus', CHAIN_AFTER, 'pointer to focus')
+    self.chainTask('braille pointer to por', CHAIN_AFTER, 'pointer to por')
     
   def close(self):
     try:
@@ -182,7 +192,8 @@ class BasicBraillePerk(Perk.Perk):
       pass
                                                      
   def getDescription(self):
-    return _('Manages basic Braille input and output')
+    return _('Manages basic screen reader Braille input and output. Requires '
+             'a loaded Braille device.')
   
   def getName(self):
     return _('Basic Braille')
@@ -195,7 +206,7 @@ class BasicBraillePerk(Perk.Perk):
       self.abscaretpos = pos
     else:
       self.abscaretpos = -1 
-      
+            
   def getRelCaretPos(self, totalcells):
     '''
     Get relative (to device display) caret position
@@ -204,7 +215,6 @@ class BasicBraillePerk(Perk.Perk):
     @rtype: Integer
     '''
     # -1 (0 on device) is special case where we don't show cursor
-    
     if self.abscaretpos == -1:
       return -1
     else:
@@ -219,7 +229,7 @@ class BasicBraillePerk(Perk.Perk):
     @rtype: int
     '''  
     state = self.getState()
-    misscellcnt = self.send(CMD_GET_MISSINGCELL_COUNT, None, talk=False)
+    misscellcnt = self.send(CMD_GET_MISSINGCELL_COUNT, None)
     
     if self.task_por.char_offset-self.pre_left_slice < state.LeftPadding:
       retval = max(0,self.task_por.char_offset-state.LeftPadding)
@@ -360,8 +370,7 @@ class HandlePageForward(Task.InputTask):
     state=self.perk.getState()
     
     ctlen = len(self.perk.current_text)
-    l_el_len, r_el_len = self.send(CMD_GET_ELLIPSIS_SIZE, None, talk=False)
-    l_el_len, r_el_len = self.send(CMD_GET_ELLIPSIS_SIZE, None, talk=False)
+    l_el_len, r_el_len = self.send(CMD_GET_ELLIPSIS_SIZE, None)
     
     if  2*style.TotalCells+self.perk.pre_left_slice+ \
            self.perk.scroll_offset-state.Overlap < ctlen + state.RightPadding:
@@ -401,9 +410,8 @@ class HandlePageBackward(Task.InputTask):
       return
     state=self.perk.getState()
     
-    l_el_len, r_el_len = self.send(CMD_GET_ELLIPSIS_SIZE, None, talk=False)
+    l_el_len, r_el_len = self.send(CMD_GET_ELLIPSIS_SIZE, None)
     
-    # TODO change hardcoded len(ellipsis)
     if self.perk.pre_left_slice+self.perk.scroll_offset- \
                      (style.TotalCells-state.Overlap) > 0:
       self.perk.scroll_offset -= \
@@ -587,20 +595,19 @@ class OutputText(Task.Task):
    
     # set caret position. 
     caretpos = self.perk.getRelCaretPos(style.TotalCells)
-  
     if caretpos < 0 or caretpos >= style.TotalCells:
-      self.send(CMD_CARET, -1, talk=False)
+      self.send(CMD_CARET, -1)
     else:
-      self.send(CMD_CARET, caretpos, talk=False)
+      self.send(CMD_CARET, caretpos)
   
     leftslice = self.perk.getLeftSlice(style.TotalCells)
     #Note: rightslice >= len(current_text) does not throw exception
     rightslice = leftslice+style.TotalCells
     
     # truncation indicators for ellipsis
-    misscellcnt = self.send(CMD_GET_MISSINGCELL_COUNT, None, talk=False)
+    misscellcnt = self.send(CMD_GET_MISSINGCELL_COUNT, None)
     self.send(CMD_TRUNCATE, (leftslice>0, 
-        rightslice<len(self.perk.current_text)+misscellcnt), talk=False) 
+        rightslice<len(self.perk.current_text)+misscellcnt)) 
         
     # send text to device
     self.sayItem(text=self.perk.current_text[leftslice:rightslice])
@@ -609,16 +616,38 @@ class HandleReview(Task.InputTask):
   '''  
   Synchronizes the Braille display with the text at the pointer when reviewing.
   
-  @todo: PP: render caret position when appropriate
   '''
   def execute(self, **kwargs):
-    result = self.getTempVal('review')
-    if result in (REVIEW_OK, REVIEW_WRAP):
-      # update current text
-      self.perk.current_text = self.getItemText()
-      self.perk.scroll_offset = self.task_por.char_offset
-      self.doTask('braille current text')
-
+    if self.perk.getState().BrailleReview:
+      result = self.getTempVal('review')
+      if result in (REVIEW_OK, REVIEW_WRAP):
+        # update current text
+        self.perk.current_text = self.getItemText()
+        # get length of ellipsis
+        l_el_len, r_el_len = self.send(CMD_GET_ELLIPSIS_SIZE, None)
+        
+        if self.perk_state.SnapOnUpdate:
+          self.perk.scroll_offset = 0
+        # get the por at the focus
+        focus_por = self.getAccFocus()
+        if focus_por is not None and (focus_por.item_offset == self.task_por.item_offset):
+          self.perk.setAbsCaretPos(focus_por.char_offset)
+        else:
+          self.perk.setAbsCaretPos(-1)
+    
+        self.perk.setAbsCaretPos(self.task_por.char_offset)
+          
+        if self.task_por.char_offset == 0:
+          self.perk.pre_left_slice = self.task_por.char_offset
+        elif self.task_por.char_offset - l_el_len >= 0:
+          self.perk.pre_left_slice = self.task_por.char_offset - l_el_len
+        else: # self.task_por.char_offset - l_el_len < 0
+          # Don't move viewport when less than ellipsis size
+          pass
+             
+        self.doTask('braille current text')
+      
+        
 class HandlePointerToFocus(Task.InputTask):
   '''
   Synchronizes the Braille display with the text at the focus when the pointer
@@ -632,4 +661,17 @@ class HandlePointerToFocus(Task.InputTask):
     if self.perk_state.SnapOnUpdate:
       self.perk.scroll_offset = 0
     # now Braille it
-    self.doTask('braille current text')
\ No newline at end of file
+    self.doTask('braille current text')
+    
+class BrailleItemText(Task.Task):
+  '''
+  Outputs item text to Braille display.
+  
+  '''
+  def execute(self, **kwargs):
+    # update current text
+    self.perk.current_text = self.getItemText()
+    if self.perk_state.SnapOnUpdate:
+      self.perk.scroll_offset = 0
+    # now Braille it
+    self.doTask('braille current text')    
\ No newline at end of file
