@@ -31,6 +31,8 @@ class BasicSpeechPerkState(Perk.PerkState):
   WordEcho (bool): Echo words when typing?
   
   CharEcho (bool): Echo characters when typing?
+  
+  AutoLang (bool): Switch languages automatically?
   '''
   def init(self):
     self.newBool('WordEcho', False, _('Echo words?'), 
@@ -42,7 +44,11 @@ class BasicSpeechPerkState(Perk.PerkState):
                  _('When set, the role name of every widget is announced. '
                    'Otherwise, the role name is only announced when a new '
                    'role is encountered.'))
-  
+    self.newBool('AutoLang', True, _('Switch languages automatically?'),
+                 _('When set, the language of the speech synthesizer tries to '
+                   'switch to the appropriate language and dialect '
+                   'automatically based on the current locale.'))
+
   def getGroups(self):
     '''
     Gets configurable settings for this L{Perk}.
@@ -51,10 +57,12 @@ class BasicSpeechPerkState(Perk.PerkState):
     @rtype: L{AEState.Setting.Group}
     '''
     g = self.newGroup()
-    e = g.newGroup(_('Echo'))
-    e.extend(['CharEcho', 'WordEcho'])
-    v = g.newGroup(_('Verbosity'))
-    v.extend(['RepeatRole'])
+    s = g.newGroup(_('Echo'))
+    s.extend(['CharEcho', 'WordEcho'])
+    s = g.newGroup(_('Verbosity'))
+    s.extend(['RepeatRole'])
+    s = g.newGroup(_('Language'))
+    s.extend(['AutoLang'])
     return g
   
 class BasicSpeechPerk(Perk.Perk):
@@ -108,7 +116,7 @@ class BasicSpeechPerk(Perk.Perk):
     self.registerTask(ReadReviewSkip('read review skip'))
     self.registerTask(ReadReviewWord('read review word'))
     self.registerTask(ReadReviewChar('read review char'))
-    self.registerTask(ReadPointerToFocus('read pointer to focus'))
+    self.registerTask(ReadPointerToFocus('read pointer to por'))
     self.registerTask(WhereAmI('where am i'))
     self.registerTask(ReadTop('read top'))
     self.registerTask(ReadTextColor('read text color'))
@@ -161,8 +169,8 @@ class BasicSpeechPerk(Perk.Perk):
     self.chainTask('read review char', AEConstants.CHAIN_AFTER, 
                    'review next char')
     
-    self.chainTask('read pointer to focus', AEConstants.CHAIN_AFTER,
-                   'pointer to focus')
+    self.chainTask('read pointer to por', AEConstants.CHAIN_AFTER,
+                   'pointer to por')
     
     self.chainTask('read review skip', AEConstants.CHAIN_AFTER,
                    'review skip report')
@@ -217,6 +225,36 @@ class BasicSpeechPerk(Perk.Perk):
     @rtype: string
     '''
     return _('Basic speech')
+  
+  def getDescription(self):
+    '''
+    @return: Human readable description of this L{Perk}.
+    @rtype: string
+    '''
+    return _('Manages basic screen reader speech output. Requires a loaded '
+             'speech device.')
+  
+  def switchLanguages(self):
+    '''
+    Attempts to switch the active speech engine to the POSIX message language
+    in use in this L{Tier}.
+    '''
+    if not self.getPerkSettingVal('AutoLang'):
+      # do not switch language automatically if the user has disabled the
+      # feature
+      return
+    # get the local string from the application
+    l = self.getAccAppLocale()
+    if l is None:
+      # no locale information provided
+      return
+    # convert the string to IANA format for the speech devices
+    tag = self.convertPOSIXToIANA(l)
+    # ask the active device to find a language close to the one specified
+    tag = self.getClosestLang(tag)
+    if tag is not None:
+      # set the tag as the active language for all voices on the device
+      self.setStyleVal('Language', tag)
 
 class Stop(Task.InputTask):
   '''
@@ -285,8 +323,10 @@ class ReadReviewSkip(Task.InputTask):
   Reads a summary of an item skipped during review.
   '''
   def execute(self, **kwargs):
-    # just say the role as the summary
-    self.sayRole()
+    if not self.getTempVal('has skipped'):
+      # be sure to stop speech if we haven't skipped other items yet
+      self.stopNow()
+    self.doTask('read new role')
     
 class ReadReviewWord(Task.InputTask):
   '''  
@@ -388,7 +428,7 @@ class ReadPointerToFocus(Task.InputTask):
   focus, selection, and caret position.
   '''
   def execute(self, **kwargs):
-    self.doTask('default focus', por=self.task_por, gained=True)
+    self.doTask('read focus', por=self.task_por, gained=True)
     self.doTask('read item details')
     
 class WhereAmI(Task.InputTask):
@@ -594,10 +634,10 @@ class ReadNewContainer(Task.Task):
       if (prev and self.hasAccRole('label', prev) and 
           not self.getAccRelations('label for', prev)):
         to_check.append(prev)
+      last = self.perk.last_container
       for acc in to_check:
         name_text = self.getAccName(acc)
         label_text = self.getAccLabel(acc)
-        last = self.perk.last_container
         if (((name_text and last.startswith(name_text)) or
             (label_text and last.startswith(label_text)) or
             self.hasAccRole('menu bar', acc))):
@@ -635,8 +675,8 @@ class ReadAll(Task.InputTask):
   '''
   def execute(self, interval=None, **kwargs):
     self.stopNow()
-    self.registerTask(_ContinueReadAll(1000, 'continuous read'))
-    self.doTask('continuous read')
+    self.registerTask(_ContinueReadAll('read all continued', 1000))
+    self.doTask('read all continued')
 
 class _ContinueReadAll(Task.TimerTask):
   '''
@@ -664,11 +704,11 @@ class _ContinueReadAll(Task.TimerTask):
       self.task_por = self.continue_por or self.task_por
       # continue where we left off
       for i in xrange(5):
-        # only invoke the review task, not all other chained to it
-        self.doTask('review next item', chain=False)
         # always indicate items have been skipped so we don't stop the speech
         # announcing them
         self.setTempVal('has skipped', True)
+        # only invoke the review task, not all other chained to it
+        self.doTask('review next item', chain=False)
         # now invoke the speech task for reviewing items
         self.doTask('read review item', chain=False, stop=False)
         if self.getTempVal('review') == AEConstants.REVIEW_NO_NEXT_ITEM:
@@ -812,10 +852,11 @@ class ReadItemDetails(Task.Task):
 class HandleViewChange(Task.ViewTask):
   '''
   Task that handles a L{AEEvent.ViewChange}.
-  
-  @todo: PP: Pause before saying 'No view'
   '''
-  def executeLost(self, por, title, **kwargs):  
+  def executeLost(self, por, title, **kwargs): 
+    '''
+    @todo: PP: Pause before saying 'No view'
+    '''
     return True
   
   def executeFirstGained(self, por, title, **kwargs):
@@ -844,12 +885,14 @@ class HandleViewChange(Task.ViewTask):
     '''
     # stop output
     self.stopNow()
+    # switch to a language matching the application locale
+    self.perk.switchLanguages()
     rv = self.executeFirstGained(por, **kwargs)
     # prevent the next event (focus?) from stopping this announcement
     self.inhibitMayStop()
     # see if we want to handle this new view in a special way
     if self.hasAccRole('alert', por):
-      self.registerTask(ReadAlert())
+      self.registerTask(ReadAlert('read alert'))
     return rv
     
 class HandleFocusChange(Task.FocusTask):
@@ -899,16 +942,28 @@ class HandleSelectorChange(Task.SelectorTask):
     Announces the localized word selected or unselected followed by the total 
     number of characters in the selection.
     
+    Single line text boxes fire selection events followed by a caret event.  
+    Multiline text boxes fire events in the opposite order.  The
+    inhibitMayStop() allows the selection event to complete for single line
+    text boxes before the caret event stops it with a call to mayStop().
+    
     @param por: Point of regard where the selection event occurred
     @type por: L{POR}
     @param text: The currently selected text
     @type text: string
     '''
+    if self.getAccCaret() != self.getPointerPOR():
+      # if the caret is out of sync with our pointer, we haven't received a
+      # movement event yet corresponding to this selection; inhibit the next
+      # may stop as the next event will be the movement
+      self.inhibitMayStop()
     n = len(text)
-    if n > self.perk.last_sel_len:
-      self.sayItem(text=n, template=_('selected %d'))
-    elif n != 0:
-      self.sayItem(text=n, template=_('unselected %d'))
+    if n >= self.perk.last_sel_len:
+      self.sayTextAttrs(text=n, template=_('selected %d'))
+    elif self.perk.last_sel_len != 0:
+      # don't say unselection when the last selection length was zero also
+      self.sayTextAttrs(text=(self.perk.last_sel_len-n, n),\
+                   template=_('unselected %d, %d remaining'))
     self.perk.last_sel_len = n
 
 class HandleCaretChange(Task.CaretTask):
@@ -958,7 +1013,6 @@ class HandleCaretChange(Task.CaretTask):
     @param text_offset: The offset of the inserted text
     @type text_offset: integer
     '''
-    #print 'CHANGED: insert'
     self.changed = True
     if len(text) == 1:
       w = (self.perk_state.WordEcho and 
@@ -995,7 +1049,6 @@ class HandleCaretChange(Task.CaretTask):
     '''
     # don't echo again if a change just happened
     if self.changed:
-      #print 'UNCHANGED: moved'
       self.changed = False
       return
     
@@ -1003,8 +1056,11 @@ class HandleCaretChange(Task.CaretTask):
       # say the current line
       self.mayStop()
       self.sayItem(text=text)
-      if self.getAccTextSelectionCount() > 0:
-        # and the total number of selected characters if there's a selection
+      if (self.hasAccState('multi line') and 
+          self.getAccTextSelectionCount() > 0):
+        # announce the selection if this is a multiline box as we often do not
+        # receive an event; klude that probably breaks for non-gtk apps
+        # @todo: PP: fix me
         self.sayTextAttrs(text=len(self.getAccTextSelection()[0]),
                           template=_('selected %s'))
     elif not por.isSameItem(self.perk.last_caret):
@@ -1012,7 +1068,6 @@ class HandleCaretChange(Task.CaretTask):
       self.mayStop()
       self.sayItem(text=text)
     else:
-      #print 'move:', por, self.perk.last_caret
       # respond to caret movement on same line
       if por.isCharBefore(self.perk.last_caret):
         # moved left, say text between positions
@@ -1024,7 +1079,8 @@ class HandleCaretChange(Task.CaretTask):
         diff = text[self.perk.last_caret.char_offset : por.char_offset]
         self.mayStop()
         self.sayItem(text=diff)
-    
+    self.may_stop = 0
+   
   def executeDeleted(self, por, text, text_offset, **kwargs):
     '''
     Announces the deleted text. Prepends delete and backspace text to indicate
@@ -1043,7 +1099,6 @@ class HandleCaretChange(Task.CaretTask):
       # say backspace if we're backing up
       self.sayItem(text=text, template=_('backspace %s'))
       self.changed = True
-      #print 'CHANGED: delete'
     else:
       # say delete if we're deleting the current
       self.sayItem(text=text, template=_('delete %s'))
@@ -1070,7 +1125,6 @@ class HandleCaretChange(Task.CaretTask):
     # store for next comparison
     self.perk.last_caret = por
     self.changed = False
-    #print 'UNCHANGED: update'
   
 class HandleStateChange(Task.StateTask):
   '''  
@@ -1093,7 +1147,8 @@ class HandleStateChange(Task.StateTask):
       # try a may stop
       self.mayStop()
       self.sayState(text=text)
-    return True
+    # don't update the pointer on a state change
+    self.inhibitAutoPointer()
 
 class HandlePropertyChange(Task.PropertyTask):
   '''
@@ -1116,6 +1171,8 @@ class HandlePropertyChange(Task.PropertyTask):
         # immediately after saying the same thing
         self.mayStop()
         self.sayItem(text=value)
+    # don't update the pointer on a property change
+    self.inhibitAutoPointer()
     
 class ReadAlert(Task.FocusTask):
   '''
