@@ -15,17 +15,15 @@ available under the terms of the BSD license which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-from AEConstants import HOME_USER, BUILTIN_PROFILES
+from AEConstants import HOME_USER, BUILTIN_PROFILES, initUserPath
 from i18n import _
 import shelve, os, shutil, whichdb
 
 # paths to parts of the settings repository
 PROFILES_PATH = os.path.join(HOME_USER, 'profiles')
 # make sure the profile folder is available
-try:
-  os.makedirs(PROFILES_PATH)
-except OSError:
-  pass
+if initUserPath(PROFILES_PATH):
+  print 'created ~/.lsr/profiles'
 
 class SettingsManager(object):
   '''
