@@ -6,15 +6,16 @@ derive.
 @author: Pete Brunet
 @author: Larry Weiss
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-import LSRConstants, LSRInterfaces
+import weakref
+import AEConstants, LSRInterfaces
 from i18n import _
 
 class TaskTools(object):
@@ -40,8 +41,7 @@ class TaskTools(object):
   @type tier: L{Tier}
   @ivar perk: The L{Perk} owning the L{Task} subclass
   @type perk: L{Perk}
-  @ivar layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
-    L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+  @ivar layer: Layer on which the event occurred
   @type layer: integer
   @ivar lsr_state: The state object shared across all L{Tier}s
   @type lsr_state: L{AEState.AEState}
@@ -51,12 +51,11 @@ class TaskTools(object):
   @ivar task_por:  The L{POR} for the Task or Perk subclass, set by 
     L{preExecute}
   @type task_por: L{POR}
-  @ivar timestamp: Time at which an event occurred. Not guaranteed to be 
-    delivered for all events nor to be based on the same absolute zero across
-    all devices.
-  @type timestamp: float
   @ivar tools_initialized: Is the instance ready to be executed?
   @type tools_initialized: boolean
+  @ivar def_out: Default output device to use for all L{Task.Tools.Output.say}
+    calls in this L{Perk} module.
+  @type def_out: weakref.proxy to L{AEOutput}
   '''
   def __init__(self):
     '''
@@ -74,15 +73,18 @@ class TaskTools(object):
     self.tier = None
     self.layer = None
     self.acc_eng = None
-    self.timestamp = 0
+    self.def_out = None
     self.tools_initialized = False
+    self.tools_auto_pointer = True
     
   def preInit(self, ae, tier, perk=None):
-    '''
-    Initializes the references to the L{Tier} and L{AccessEngine} owning this 
-    instance of the L{TaskTools}. Contacts the L{AccessEngine} to get refs to
-    all managers. These references are fixed throughout the life of the tools 
-    object.
+    ''' 
+    Initializes the references to the L{Perk}, L{Tier}, L{AccessEngine}
+    owning this instance of the L{TaskTools}. Contacts the L{AccessEngine} to
+    get refs to all managers. These references are fixed throughout the life of
+    the tools object.
+    
+    Called before state information is loaded from disk.
     
     @param ae: Reference to AccessEngine for event context
     @type ae: L{AccessEngine}
@@ -117,8 +119,8 @@ class TaskTools(object):
   def postClose(self):
     '''
     Frees the references to the L{Tier}, L{Perk}, and L{AccessEngine} as well 
-    as all borrowed references from the L{AccessEngine}. Intended to be called
-    after L{close}.
+    as all borrowed references from the L{AccessEngine}. Releases references to
+    state information. Intended to be called after L{close}.
     '''
     if not self.tools_initialized:
       return
@@ -126,7 +128,13 @@ class TaskTools(object):
     self.perk = None
     self.acc_eng.unloanManagers(self)
     self.acc_eng = None
+    # important! throw away this weak reference else the callback keeps this
+    # object alive
+    self.def_out = None
     self.tools_initialized = False
+    # throw away references to state
+    self.lsr_state = None
+    self.perk_state = None
 
   def close(self):
     '''
@@ -134,43 +142,81 @@ class TaskTools(object):
     '''
     pass
   
-  def preExecute(self, layer, event_por=None, timestamp=0):
+  def preExecute(self, layer, event_por=None):
     '''
     Configures this L{TaskTools} object before execution.
 
-    @param layer: Layer on which the event occurred, one of 
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     @param event_por: Point of regard received with the event that triggered
       this execution
     @type event_por: L{POR}
-    @param timestamp: Time at which the event occurred
-    @type timestamp: float
     @return: Is the instance ready for execution?
     @rtype: boolean
     '''
     if not self.tools_initialized:
+      # don't do anything if the tools aren't initialized
       return False
+
     # get all state objects
+    # @todo: PP: should probably only be done once, but state is not loaded
+    # from disk until after preInit is called
     self.lsr_state = self.tier_man.getState()
     self.perk_state = self.perk.getState()
-    # store the layer and timestamp
+    
+    # reset the auto pointer flag to True
+    self.tools_auto_pointer = True
+
+    # store the layer
     self.layer = layer
-    self.timestamp = timestamp
+
     # use the POR from the event or the last saved Task POR if there is no POR 
     # associated with the event
     self.task_por = event_por or self.tier.getPointer()
+    
+    try:
+      # call a method on the default output device to ensure the ref is alive
+      self.def_out.getName()
+    except (AttributeError, ReferenceError):
+      # get an output device to serve as the default output
+      self._changeDefaultOutput()
     return True
 
-  def postExecute(self, save_por=True):
+  def postExecute(self):
     '''
-    Unconfigures this L{TaskTools} object after execution. Updates the L{Tier}
-    pointer if L{save_por} is True.
-    
-    @param save_por: Store the L{task_por} as the pointer in the L{Tier}?
-    @type save_por: boolean
+    Unconfigures this L{TaskTools} object after execution. Automatically 
+    updates the L{Tier} pointer L{POR} if the subclass is not the L{Perk},
+    the L{tools_initialized} flag is True, the L{tools_auto_pointer} is True,
+    and the event handled is in the focus layer.
     '''
-    if save_por and self.tools_initialized:
+    if (self.perk is not self and # never store POR from Perk objects
+        self.tools_initialized and # only store if initialized
+        self.tools_auto_pointer and # only store if auto pointer
+        AEConstants.LAYER_FOCUS == self.layer): # only store if focus layer
       self.tier.setPointer(self.task_por)
-    self.lsr_state = None
-    self.perk_state = None
+    # PP: removed these because recursive nature of Tasks can 
+    # postExecute a Perk that is still executing and it will lose its state
+    # references (yikes)
+    #self.lsr_state = None
+    #self.perk_state = None
+    #self.def_out = None
+
+  def _changeDefaultOutput(self, ref=None):
+    '''
+    Gets an output device based on the ideal capabilities requested by the
+    current L{Perk}. Creates a weak proxy for the device object. When the
+    weak ref is no longer valid, this method is called again to find another
+    suitable device.
+    
+    @param ref: Weak proxy that is no longer valid
+    @type ref: weakref.proxy
+    '''
+    if not self.tools_initialized:
+      return
+    # get an output device to serve as the default output
+    caps = self.perk.getIdealOutput()
+    dev = self.device_man.getOutputByCaps(caps)
+    try:
+      self.def_out = weakref.proxy(dev, self._changeDefaultOutput)
+    except TypeError:
+      self.def_out = None
