@@ -3,7 +3,7 @@ Defines L{Tools} for getting input.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
@@ -19,18 +19,30 @@ class Input(Base.TaskTools):
   '''
   Provides methods for getting input on any L{AEInput} device.
   '''
-  def getInputDevice(self, name):
+  def getInputDevice(self, name=None, *capabilities):
     '''
-    Gets an L{AEInput} device from the L{DeviceManager} given its name.
+    Gets an L{AEOutput} device from the L{DeviceManager} given its name or
+    its capabilities. The capabilities list may include strings naming 
+    L{AEInput} interfaces ("braille" and/or "system input" at present).
     
-    @param name: UIE name of the L{AEInput} device to get
+    If neither is specified, the first available output device in the
+    L{DeviceManager} is returned.
+    
+    @param name: Class (UIE) name of the L{AEInput} device to get
     @type name: string
-    @return: The named input device
+    @param capabilities: Names of capabilities required on the device
+    @type capabilities: list of string
+    @return: The named output device
     @rtype: L{AEInput}
-    @raise InvalidDeviceError: When a L{AEInput} device with the given name is
-      not registered
+    @raise InvalidDeviceError: When the specified L{AEInput} device is not 
+      found
     '''
-    device = self.device_man.getInputByName(name)
+    if name is not None:
+      device = self.device_man.getInputByName(name)
+    elif len(capabilities):
+      device = self.device_man.getInputByCaps(capabilities)
+    else: 
+      device = self.device_man.getFirstInput()
     if device is None:
       # let the system report the device isn't available
       raise InvalidDeviceError
