@@ -15,7 +15,7 @@ available under the terms of the BSD license which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-import logging, weakref
+import logging, weakref, traceback, sys, os
 import AEState, AEInput, Perk, Task, AEConstants
 from POR import POR
 
@@ -55,10 +55,17 @@ class Tier(object):
   @ivar chain_refs: References to L{Perk}s keyed by task names. Many to many
     mapping. See L{addChainRef} and L{removeChainRef}.
   @type chain_refs: dictionary
+  @ivar chain_stack: Stack of L{Task} and L{Perk} names currently executing in 
+    one or more chains before, after, or around one or more anchors
+  @type chain_stack: list
   @ivar temp_data: Arbitrary name/value pairs stored by L{Task}s executing in
     this L{Tier} in response to a single event. The data is persistent until 
     the event has been handled by all L{Task}s at which time it is cleared.
   @type temp_data: dictionary
+  @ivar task_history: Stack of L{Task} and L{Perk} names having already 
+    executed in response to an L{AEEvent}, including all chained L{Task}s and
+    all L{Task}s executed from within other L{Task}s
+  @type task_history: list
   '''
   def __init__(self, tier_manager, name, aid, por):
     '''
@@ -86,8 +93,9 @@ class Tier(object):
     self.chain_refs = {}
     self.pointer_por = por
     self.focus_por = None
-    self.last_key = (None, None)
-    self.temp_data = {}
+    self.last_key = (None, None, None)
+    # fields to be cleared after each event
+    self.clearState()
 
   def close(self):
     '''
@@ -122,6 +130,30 @@ class Tier(object):
     '''
     return self.focus_por
   
+  def getHistory(self):
+    '''
+    Gets the entire linear history of L{Task}s that have executed and the 
+    L{Perk}s they are in.
+    
+    @return: List of L{Task} identities and L{Perk} class names
+    @rtype: list of 2-tuple of string
+    '''
+    return self.task_history
+  
+  def getAnchor(self):
+    '''
+    Gets the identity of the L{Task} to which the current L{Task} is chained 
+    and its corresponding L{Perk}. If there is no anchor for the executing 
+    L{Task}, this method returns None.
+    
+    @return: L{Task} identity and L{Perk} class name or None
+    @rtype: 2-tuple of string
+    '''
+    try:
+      return self.chain_stack[0]
+    except IndexError:
+      return None
+  
   def getLastKey(self):
     '''    
     Gets the last key code, key sym, and key modifiers. Useful for dealing with
@@ -155,11 +187,14 @@ class Tier(object):
     '''
     self.temp_data[name] = value
     
-  def clearChainData(self):
+  def clearState(self):
     '''
-    Clears all stored L{temp_data}.
+    Clears all stored L{temp_data}, the L{chain_stack}, and the 
+    L{task_history}.
     '''
     self.temp_data = {}
+    self.task_history = []
+    self.chain_stack = []
   
   def addChainRef(self, target, perk):
     '''
@@ -281,8 +316,11 @@ class Tier(object):
           pass
         perk.postClose()
         # warn about the error
-        log.debug('Cannot initialize %s: %s', perk.getClassName(), str(e))
-      perk.postExecute(False)
+        info = traceback.extract_tb(sys.exc_info()[2])
+        log.debug('cannot initialize %s (%s, %d): %s', perk.getClassName(), 
+                  os.path.basename(info[-1][0]),
+                  info[-1][1], str(e))
+      perk.postExecute()
     
   def popPerk(self, *indices):
     '''
@@ -325,6 +363,8 @@ class Tier(object):
     '''
     Gets all L{Perk}s pushed onto this L{Tier} in execution order.
     
+    @note: This method does not return a copy of the stack. Modifications will
+      affect this L{Tier}!
     @return: All L{Perk}s pushed onto this L{Tier}
     @rtype: list of L{Perk}
     '''
@@ -506,9 +546,11 @@ class Tier(object):
     if rv is None:
       # continue processing if no return value is specified
       rv = True
-    # uninitialize the Task, potentially storing the POR only if the event was
-    # in the focus layer and execution is still propagating
-    task.postExecute(task.layer == AEConstants.LAYER_FOCUS and propagate)
+    elif not rv:
+      # inhibit automatic updating of the pointer if propogation is blocked
+      task.inhibitAutoPointer()
+    # uninitialize the Task
+    task.postExecute()
     return rv
   
   def manageEvent(self, event):
@@ -561,10 +603,20 @@ class Tier(object):
         continue
       # run through all Tasks of the given type in this Perk
       for task in perk.getEventTasks(event_type, task_layer):
+        try:
+          i = task.getIdentity()
+        except ValueError:
+          i = None
+        # add to linear history before executing so its available to chained
+        # and doTask tasks
+        self.task_history.append((i, perk.getClassName()))
+        
+        # execute task and log information
+        self.tier_manager.showTask(task, perk)
         propagate = self._executeTask(task, event_por, task_layer, 
                                       task_params, propagate)
-        self.tier_manager.showTask(event, perk, task, propagate)
-      perk.postExecute(False)
+        self.tier_manager.showPropagate(propagate)
+      perk.postExecute()
         
   def manageGesture(self, event, count):
     '''
@@ -612,9 +664,11 @@ class Tier(object):
       task_params = event.getDataForTask()
       if not perk or not perk.preExecute(layer):
         return
-      # execute that Task
-      self._executeTask(task, None, layer, task_params, True)
-      perk.postExecute(False)
+      # execute that Task and log its information
+      self.tier_manager.showTask(task, perk)
+      rv = self._executeTask(task, None, layer, task_params, True)
+      self.tier_manager.showPropagate(rv)
+      perk.postExecute()
 
   def manageNamedTask(self, por, name, layer, task_params, propagate=True,
                       chain=True):
@@ -622,10 +676,7 @@ class Tier(object):
     Executes a L{Task} registered under the given name in this L{Tier} by
     configuring it to run in the given layer and with the given keyword 
     argument data. Follows any chained L{Task}s and recurisvely executes them 
-    before, around, and after. 
-    
-    @todo: PP: show doTask? self.tier_manager.showEvent(event, self.name)
-    @todo: PP: implement support for around
+    before, around, and after.
     
     @param name: Name of the L{Task} to execute
     @type name: string
@@ -645,29 +696,74 @@ class Tier(object):
     task = self.getNamedTask(name)
     # bail immediately if it doesn't exist
     if task is None: return
+    perk = task.getPerk()
+    # put the name of this task and its perk onto the stack
+    self.chain_stack.append((name, perk.getClassName()))
     
     if chain:
       # get the chain of tasks to run before this one
       for before_name in self.getChainedTasks(name, AEConstants.CHAIN_BEFORE):
+        self.tier_manager.showChain(AEConstants.CHAIN_BEFORE, name)
         # recurse with before tasks
-        self.manageNamedTask(por, before_name, layer, task_params, propagate)
-        # use the pointer as the input to the next task
+        propagate = self.manageNamedTask(por, before_name, layer, task_params, 
+                                         propagate)
+        
+    if chain:
+      # check for task to run instead of this one
+      around_name = self.getChainedTasks(name, AEConstants.CHAIN_AROUND)
+      if around_name is None or len(around_name) == 0:
+        perk = task.getPerk()
+        if perk.preExecute(layer, por):
+          # pop the name of this task off the stack
+          cache = self.chain_stack.pop()
+          # add to the linear history before executing so that it's available
+          # to changed and doTask tasks
+          self.task_history.append((name, perk.getClassName()))
+          # execute this Task and log information
+          self.tier_manager.showTask(task, perk)
+          propagate = self._executeTask(task, por, layer, task_params, 
+                                        propagate)
+          self.tier_manager.showPropagate(propagate)
+          perk.postExecute()
+          # get the pointer as the starting POR for the next task execution
+          por = self.getPointer()
+          # push the name of this task back onto the stack
+          self.chain_stack.append(cache)
+      else:
+        self.tier_manager.showChain(AEConstants.CHAIN_AROUND, name)
+        # recurse with around task
+        propagate = self.manageNamedTask(por, around_name[0], layer, 
+                                         task_params, propagate)
+        # get the pointer after the recursive step
+        por = self.getPointer()
+    else:
+      perk = task.getPerk()
+      if perk.preExecute(layer, por):
+        # add to the linear history before executing so that it's available
+        # to changed and doTask tasks
+        self.task_history.append((name, perk.getClassName()))
+        # execute this Task and log information
+        self.tier_manager.showTask(task, perk)
+        propagate = self._executeTask(task, por, layer, task_params, propagate)
+        self.tier_manager.showPropagate(propagate)
+        perk.postExecute()
+        # should this go before or after _executeTask
         por = self.getPointer()
-      
-    perk = task.getPerk()
-    if perk.preExecute(layer, por):
-      # execute this Task
-      self._executeTask(task, por, layer, task_params, propagate)
-      perk.postExecute(False)
     
     if chain:
       # get the chain of tasks to run after this one
       for after_name in self.getChainedTasks(name, AEConstants.CHAIN_AFTER):
-        # recurse with before tasks
-        self.manageNamedTask(por, after_name, layer, task_params, propagate)
-        # use the pointer as the input to the next task
-        por = self.getPointer()
-
+        self.tier_manager.showChain(AEConstants.CHAIN_AFTER, name)
+        # recurse with after tasks
+        propagate = self.manageNamedTask(por, after_name, layer, task_params, 
+                                         propagate)
+        
+    # pop the name of this task off the stack permanently
+    self.chain_stack.pop()
+    
+    # return the current value of propagate
+    return propagate
+        
   def managePrivate(self, event):
     '''
     Pulls private data from an event and stores it in appropriate instance
