@@ -159,29 +159,21 @@ class TierManager(object):
     '''
     self.monitors.add(AEEvent.Base.AccessEngineEvent, monitors)
         
-  def showTask(self, event, perk, task, propogate):
+  def showTask(self, task, perk):
     '''    
-    Informs L{AEMonitor}s added via L{addMonitors} of a L{Task} handling an 
-    event. Calls show on the monitors collection as soon as the L{Task} has 
-    finished executing.
+    Informs L{AEMonitor}s added via L{addMonitors} of a L{Task} executing or
+    updating.
     
-    @param event: An L{AEEvent} object
-    @type event: L{AEEvent}
-    @param perk: L{Perk} that handled the event
+    @param perk: L{Perk} handling the event
     @type perk: L{Perk}
-    @param task: L{Task} that handled the event
+    @param task: L{Task} handling the event
     @type task: L{Task}
-    @param propogate: Was the event consumed (False) or allowed to propogate 
-      (True)?
-    @type propogate: boolean
     '''
-    self.monitors.show(event=event, perk=perk, task=task, propogate=propogate)
+    self.monitors.show(task=task, perk=perk)
         
   def showEvent(self, event, tier_name):
     '''    
-    Informs L{AEMonitor}s added via L{addMonitors} of an event. Calls
-    show on the monitors collection as soon as an event is accepted for
-    processing by a L{Tier}.
+    Informs L{AEMonitor}s added via L{addMonitors} of an event.
     
     @param event: An L{AEEvent} object
     @type event: L{AEEvent}
@@ -190,6 +182,29 @@ class TierManager(object):
     '''
     self.monitors.show(event=event, tier_name=tier_name)
     
+  def showChain(self, chain_type, anchor_ident):
+    '''    
+    Informs L{AEMonitor}s added via L{addMonitors} of a chained L{Task} 
+    executing.
+    
+    @param chain_type: One of the L{AEConstants} CHAIN_* constants
+    @type chain_type: integer
+    @param anchor_ident: Identifier of the anchor L{Task}
+    @type anchor_ident: string
+    '''
+    self.monitors.show(chain_type=chain_type, anchor_ident=anchor_ident)
+    
+  def showPropagate(self, propagate):
+    '''    
+    Informs L{AEMonitor}s added via L{addMonitors} of the propagation return
+    value from a L{Task}.
+    
+    @param propagate: Was the event consumed (False) or allowed to propogate 
+      (True)?
+    @type propagate: boolean
+    '''
+    self.monitors.show(propagate=propagate)
+    
   def freeDeadTiers(self, aids):
     '''
     Compares the list of given application names and IDs with those currently
@@ -334,7 +349,7 @@ class TierManager(object):
       event.setLayer(AEConstants.LAYER_TIER)
     # let the tier manage the event
     tier.manageEvent(event)
-    tier.clearChainData()
+    tier.clearState()
    
   def manageGesture(self, event):
     '''
@@ -355,7 +370,7 @@ class TierManager(object):
         self.gesture[1] = 0
       # let the active Tier manage the event
       self.active_tier.manageGesture(event, self.gesture[1])
-      self.active_tier.clearChainData()
+      self.active_tier.clearState()
       
   def manageChooser(self, event):
     '''
@@ -375,7 +390,7 @@ class TierManager(object):
       return
     # let that Tier manage the event
     tier.manageKeyedTask(event)
-    tier.clearChainData()
+    tier.clearState()
     
   def manageTimer(self, event):
     '''
@@ -397,7 +412,7 @@ class TierManager(object):
       event.setLayer(AEConstants.LAYER_TIER)
     # let that Tier manage the event
     tier.manageKeyedTask(event) 
-    tier.clearChainData()
+    tier.clearState()
 
   def managePrivate(self, event):
     '''
@@ -410,4 +425,4 @@ class TierManager(object):
     '''
     if self.active_tier is not None:
       self.active_tier.managePrivate(event)
-      self.active_tier.clearChainData()
\ No newline at end of file
+      self.active_tier.clearState()
\ No newline at end of file
