@@ -59,7 +59,7 @@ available under the terms of the BSD license which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-import os, glob, imp, cPickle, sys, logging, shutil
+import os, glob, imp, cPickle, sys, logging, shutil, traceback
 from SettingsManager import SettingsManager
 from i18n import _
 from AEConstants import HOME_USER, HOME_DIR, PROG_VERSION, PROG_DATE, \
@@ -809,9 +809,14 @@ def loadOne(name):
   '''
   try:
     return _getUIEInstance(name)
+  except SyntaxError, e:
+    # on any exception, pass on loading this UIE, but log the error
+    log.error('syntax error in %s: %s', name, str(e))
+    return None    
   except Exception, e:
     # on any exception, pass on loading this UIE, but log the error
-    log.debug('Cannot load %s: %s', name, str(e))
+    info = traceback.extract_tb(sys.exc_info()[2])
+    log.debug('cannot load %s (%d): %s', name, info[-1][1], str(e))
     return None
 
 def loadAssociated(kind, profile, tier=None):
